
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: //:optimeering

from setuptools import setup

setup(**{
    'author': 'Optimeering',
    'classifiers': [
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    'description': 'Optimeering Python Client',
    'install_requires': (
        'azure-identity<2.0.0,>=1.17.1',
        'orjson<4.0.0,>=3.10.15',
        'pydantic>=2',
        'python-dateutil>=2.8.2',
        'types-python-dateutil>=2.8.19.14',
        'typing-extensions>=4.7.1',
        'urllib3>=1.25.3',
    ),
    'long_description': '# optimeering-python-sdk',
    'long_description_content_type': 'text/markdown',
    'name': 'optimeering',
    'namespace_packages': (
    ),
    'package_data': {
    },
    'packages': (
        'optimeering',
        'optimeering.api',
        'optimeering.models',
    ),
    'python_requires': '<4.0,>=3.10',
    'version': '0.0.2',
})
